/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: config_file.c,v 1.30.2.1.2.1 2002/02/25 22:20:46 lloydy Exp $
 *
 */
#include "config_file.h"
#include "server.h"
#include "chunkyircd.h"

/* Server configuration info */
char server_name[SERVNAME_MAX+1]="someserver.somehost.org";
char server_info[SERVINFO_MAX+1]="Chunky Monkey IRCD";
char load_date[40];

/* This is nasty too */
char **motd=NULL;
int mass_kicks=0;
int max_chancount=10;

/* X-Lines We Handle */
struct irc_op **olines=NULL;
static struct cline **clines=NULL;
static struct nline **nlines=NULL;

/* After load, contains the filename of the config file
 * we're using */
char curr_conf_file[200]="\0";

/* Server counters */
long int temp_count=0;
int ops=0;
long int channels=0;

/* Command Line Options */
int debug=0;
int shouldfork=0;
char *cmdline_conf_file=NULL;
int cmdline_port=0;
char *cmdline_ip=NULL;

/* Internal Parsing routines */
static int internal_parse_file(char *,int []);
static int internal_motd_load(char *);
static void internal_print_usage();
static void internal_print_version();

/*
* Parse the config file
* Really just a wrapper round internal_parse_file()
*/
int parse_config_file(char *filename, /* Config File Name */
		      int server_sockets[])
{
	time_t curr_time;
	struct tm *split_time;
	char full_path[200];
	int i;

	/* Fill socket array with -1 */
	for(i=0;i < 32;i++) 
		server_sockets[i]=-1;

	/* Set Load Date */
	time(&curr_time);
	split_time=localtime(&curr_time);
	strftime(load_date,40,"%a %d %B %Y %H:%M",split_time);

	/* Actually call routine to do parsing */
	if(cmdline_conf_file) {
		/* Use config file spec'ed on cmdline */
		internal_parse_file(cmdline_conf_file,server_sockets);
		strncpy(curr_conf_file,cmdline_conf_file,200);
	} else {
		snprintf(full_path,200,"%s/%s",CONFDIR,filename);
		internal_parse_file(full_path,server_sockets);
		strncpy(curr_conf_file,full_path,200);
	}
	
	/* Free cmdline_* vars */
	if(cmdline_conf_file)
		free(cmdline_conf_file);
	if(cmdline_ip)
		free(cmdline_ip);

	return 1;
}

/* 
* Called to reload the config file
*/
void rehash(int signal)
{
	int i;

	if(signal!=SIGHUP) {
		return;
	}
		
	/* Send Log Message */
	log_msg(LOG_BASIC,"Rehashing from %s",curr_conf_file);
	
	/* Free MOTD */
	for(i=0;motd!=NULL && motd[i]!=NULL;i++) {
		free(motd[i]);
		motd[i]=NULL;
	} 
	free(motd);
	motd=NULL;

	/* Free cline */
	for(i=0;clines!=NULL && clines[i]!=NULL;i++) {
		free(clines[i]);
		clines[i]=NULL;
	} 
	free(clines);
	clines=NULL;

	/* Free olines */
	for(i=0;olines!=NULL && olines[i]!=NULL;i++) {
		free(olines[i]);
		olines[i]=NULL;
	} 
	free(olines);
	olines=NULL;

	/* Free nlines */
	for(i=0;nlines!=NULL && nlines[i]!=NULL;i++) {
		free(nlines[i]);
		nlines[i]=NULL;
	} 
	free(nlines);
	nlines=NULL;

	/* Re-initialise modules */
	ircd_module_unload_all();
	ircd_module_rehash();
	
	/* Reread config file */
	internal_parse_file(curr_conf_file,NULL);
	/* Add our new server info into struct */
	strncpy(get_my_server()->info,server_info,SERVINFO_MAX);
	
}



/*
 * Handles the bind node
 */
static void bind_worker(char *key,char *value,int server_sockets[])
{
	static int curr_socket=0;
	static struct sockaddr_in bind_ip;

	if(!strcmp(key,"ip")) {
		inet_aton(value,&(bind_ip.sin_addr));
	}

	if(!strcmp(key,"port")) {
		bind_ip.sin_port = htons(atoi(value));
	}
			
	if(!strcmp(key,"}")) {
		/* Bind to it */
		int retval;
		retval=setup_tcp(&bind_ip);
		bind_ip.sin_port = 0;
		bind_ip.sin_addr.s_addr = 0;
		if(retval!=-1) {
			server_sockets[curr_socket++]=retval;
		}
		
	}
}

/*
 * Used to parse the main node 
 */
static void main_worker(char *key,char *value,int server_socket[])
{
	/* Stop the compiler warning */
	server_socket = server_socket;
	
	/* Only done at load time */
	if(strcmp(key,"name")==0 && curr_conf_file[0]=='\0') {
		strncpy(server_name,value,SERVNAME_MAX);
		return;
	}
			
	if(!strcmp(key,"info")) {
		strncpy(server_info,value,SERVINFO_MAX);	
		return;
	}			

	if(!strcmp(key,"maxchans")) {
		max_chancount = atoi(value);
		return;
	}
			
	if(!strcmp(key,"masskicks")) {
		if(lc_strncmp(value,"yes",-1)==0) {
			mass_kicks = 1;
		}
		return;
	}

	if(!strcmp(key,"motdfile")) {
		internal_motd_load(value);
		return;
	}	
			
	if(key[0]=='}') {
		return;
	}	
				
}

/*
 * Handles oper nodes
 */
static void oper_worker(char *key,char *value,int server_sockets[])
{
	static char pass[20]="",host[HOST_LENGTH+1]="",name[20]="";
	static int array_size=0;
	int i;

	server_sockets = server_sockets;
	
	if(!strcmp(key,"password")) {
		strcpy(pass,value);
	}

	if(!strcmp(key,"hostmask")) {
		strcpy(host,value);
	}

	if(!strcmp(key,"account")) {
		strcpy(name,value);
	}

	if(!strcmp(key,"}"))  {
		/* Allocate this nasty stuff */
		if(array_size==0 || olines==NULL)  {
			olines=calloc(2,sizeof(struct irc_op *));
			array_size=2;
		} else {
			array_size++;
			olines=realloc(olines,sizeof(struct irc_op *) * array_size);
			olines[array_size-1]=NULL;
		}
	
		for(i=0;olines[i]!=NULL;i++);
		
		olines[i]=ircd_malloc(sizeof(struct irc_op));

		strcpy(olines[i]->pass,pass);
		strcpy(olines[i]->name,name);
		strcpy(olines[i]->hostmask,host);

		pass[0]='\0';
		name[0]='\0';
		host[0]='\0';
	}

	return;	
}

/*
* Add a cline
*/
static void connect_worker(char *key,char *value,int server_sockets[])
{
	int i;
	static char pass[PASS_LENGTH+1]="",host[HOST_LENGTH]="",name[SERVNAME_MAX]="";
	static char enc_key[16];
	static int options=0,port=0;
	static int array_size=0;
	
	/* Stop gcc warning */	
	server_sockets = server_sockets;

	if(!strcmp(key,"password")) {
		strncpy(pass,value,PASS_LENGTH);
	}
	if(!strcmp(key,"host")) {
		strncpy(host,value,HOST_LENGTH);
	}
	if(!strcmp(key,"port")) {
		port = atoi(value);
	}
	if(!strcmp(key,"name")) {
		strncpy(name,value,SERVNAME_MAX);
	}
	if(!strcmp(key,"key")) {
		strncpy(enc_key,value,16);
	}
	if(!strcmp(key,"options")) {
		if(!strcmp(value,"compress")) {
			options |= USR_ZIP;
		}
	}

	if(!strcmp(key,"}")) {


		/* Allocate this nasty stuff */
		if(array_size==0 || clines==NULL)  {
			clines=calloc(2,sizeof(struct cline *));
			array_size=2;
		} else {
			array_size++;
			clines=realloc(clines,sizeof(struct cline *) * array_size);
			clines[array_size-1]=NULL;
		}
	
		for(i=0;clines[i]!=NULL;i++);
	
		clines[i]=ircd_malloc(sizeof(struct cline));

		strncpy(clines[i]->password,pass,PASS_LENGTH);
		clines[i]->port=port;
		strncpy(clines[i]->host,host,HOST_LENGTH);
		strncpy(clines[i]->serv_name,name,SERVNAME_MAX);
		memcpy(clines[i]->key,enc_key,16);
	}
	return ;	
}

/*
 * Add a nline
 */
static void accept_worker(char *key,char *value,int server_sockets[])
{
	int i;
	static char pass[PASS_LENGTH+1]="",host[HOST_LENGTH]="",name[SERVNAME_MAX]="";
	static char enc_key[16];
	static int options=0;
	static int array_size=0;

	/* Stop gcc warning */
	server_sockets = server_sockets;

	if(!strcmp(key,"password")) {
		strncpy(pass,value,PASS_LENGTH);
	}
	if(!strcmp(key,"host")) {
		strncpy(host,value,HOST_LENGTH);
	}
	if(!strcmp(key,"name")) {
		strncpy(name,value,SERVNAME_MAX);
	}
	if(!strcmp(key,"key")) {
		strncpy(enc_key,value,16);
	}
	if(!strcmp(key,"options")) {
		if(!strcmp(value,"compress")) {
			options |= USR_ZIP;
		}
	}

	if(!strcmp(key,"}")) {
		/* Allocate this nasty stuff */
		if(array_size==0 || nlines==NULL)  {
			nlines=calloc(2,sizeof(struct nline *));
			array_size=2;
		} else {
			array_size++;
			nlines=realloc(nlines,sizeof(struct nline *) * array_size);
			nlines[array_size-1]=NULL;
		}
		
		for(i=0;nlines[i]!=NULL;i++);
	
		nlines[i]=ircd_malloc(sizeof(struct nline));

		strncpy(nlines[i]->password,pass,PASS_LENGTH);
		strncpy(nlines[i]->host,host,HOST_LENGTH);
		strncpy(nlines[i]->serv_name,name,SERVNAME_MAX);
		memcpy(nlines[i]->key,enc_key,16);
	}
	return;	
}
struct config_node nodes[] = { {"bind",bind_worker},
			       {"main",main_worker},
			       {"oper",oper_worker},
			       {"connect",connect_worker},
			       {"accept",accept_worker},
			       {NULL,NULL},
			      };

/* 
* This is routine that actually does the parsing
*/
static int internal_parse_file(char *conf_file_name,
			       int server_sockets[])
{
	FILE *conf_file;
	char buffer[512];
	char *temp_curr_node=NULL;
	char curr_node[50];
	int i,done;

	conf_file = fopen(conf_file_name,"r");
	
	if(!conf_file) {
		printf("Failed to open %s\n",conf_file_name);
		exit(1);
	}
	
	while(fgets(buffer,511,conf_file)) {
		for(i=0, done=0; done==0 && buffer[i]!='\0'; i++) {
			if(buffer[i]=='{') {
				buffer[i]='\0';
				done = 1;
			}
		}

		if(done) {
			int keys_done=0;
			
			temp_curr_node = buffer;
			for(i=0;temp_curr_node[i]!='\0';i++) {
				if(temp_curr_node[i]==' ') {
					temp_curr_node[i]='\0';
				}
			}
			strcpy(curr_node,temp_curr_node);
			
			while(!keys_done) {
		
				if(fgets(buffer,511,conf_file)) {
					int done_stripping=0,done_splitting=0;
					char *stripped=NULL,*value=NULL,*key=NULL;
				
					/* Strip the tabs and spaces from the start of the string */
					stripped=buffer;
					while(!done_stripping) {

						switch(stripped[0]) {
		
							case ' ':
							case '\t':
								stripped++;
								break;

							default:
								done_stripping=1;
								break;
						}
					}
					for(i=strlen(stripped)-1;i>=0;i--) {
						switch(stripped[i]) {
							case ' ':
							case '\t':
							case '\n':
							case '\r':
								stripped[i]='\0';
								break;
							default:
								i=-2;
								break;
						}
					}		

					/* Parse the stripped text */
					key=stripped;
					while(!done_splitting) {
						switch(stripped[0]) {
							case ' ':
								value=stripped+1;
								stripped[0]='\0';
								done_splitting=1;
								break;
							case '\0':
								value=NULL;
								done_splitting=1;
								break;
							default:
								stripped++;
								break;
						}
					
					}

					for(i=0;nodes[i].name!=NULL;i++) {
						if(!strcmp(nodes[i].name,curr_node)) {
							nodes[i].worker(key,value,server_sockets);
						}
					}
					
					/* If the key is }, break out of this loop */
					if(key[0]=='}') {
						keys_done=1;
					}

				}
				
			}
				
		}
	}

	fclose(conf_file);
	return 1;
}



/*
* Load the motd 
*/
static int internal_motd_load(char *filename)
{
	
	int motd_fd=-1;
	long int curr_offset=0;
	int line_len=0;
	char buffer[1024];
	int retval=0;
	int array_size=0;
	int i,j=0;
	
	motd_fd=open(filename,O_RDONLY);

	if(motd_fd==-1) {
		printf("Failed to open MOTD file\n");
		return 0;
	}
	

	while((retval=read(motd_fd,buffer,1024))>0) {
		buffer[retval]=0;
			
		for(i=0;buffer[i]!='\0'&& line_len==0;i++) {
			if(buffer[i]=='\n') {
				line_len=i+1;
				buffer[i]='\0';
			}
		}

		if(motd==NULL) {
			motd=calloc(2,sizeof(char *));
			motd[0]=NULL;
			motd[1]=NULL;
			array_size=2;
		} else {
			array_size++;
			motd=realloc(motd,array_size * sizeof(char *));
			motd[array_size-1]=NULL;
		}

		motd[j]=string_alloc(strlen(buffer));
		strcpy(motd[j],buffer);
	
		j++;
		curr_offset += line_len;
		lseek(motd_fd,curr_offset,SEEK_SET);	
		line_len=0;
	}

	close(motd_fd);
	return 1;
}

#ifdef HAVE_GETOPT_H
/* Only valid with GNU getopt */
/* Command line params are in this array of structs */
struct option long_options[] = { {"ip",1,0,'i'},
				  {"port",1,0,'p'},
				  {"debug",1,0,'d'},
				  {"help",0,0,'h'},
				  {"config-file",1,0,'c'},
				  {"foreground",0,0,'f'},
				  {"version",0,0,'V'},
				  {NULL,0,0,0},
				};
#endif

/* 
* Command line parser
*/
int parse_cmdline(int argc,char *argv[])
{

	char retval;

#ifdef HAVE_GETOPT_H
	int option_index;
	
	while((retval=getopt_long(argc,argv,"i:p:d:fhVc:",long_options,&option_index))!=EOF) {
#else
	while((retval=getopt(argc,argv,"i:p:d:fhVc:"))!=EOF) {
#endif
		
		switch (retval) {	

			case 'i':
				cmdline_ip=string_alloc(strlen(optarg));
				strcpy(cmdline_ip,optarg);
				break;

			case 'p':
				cmdline_port=atoi(optarg);
				break;

			case 'd':
				debug=atoi(optarg);
				break;

			case 'c':
				cmdline_conf_file=string_alloc(strlen(optarg));
				strcpy(cmdline_conf_file,optarg);
				break;

			case 'f':
				shouldfork=1;
				break;

			case 'V':
				internal_print_version();
				exit(0);
				break;

			case 'h':
				internal_print_usage();
				exit(0);
				break;
		}

	}

	return 1;
}

/*
* Get a CLINE when we try and connect 
*/
struct cline *get_cline(char *serv_name)
{
	int i,done=-1;

	if(clines==NULL) {
		return NULL;
	}
	
	for(i=0;clines[i]!=NULL && done==-1;i++) {
		if((lc_strncmp(clines[i]->serv_name,serv_name,-1)==0) ) {
			done=i;
		}
	}

	if(done!=-1) {
		return clines[done];
	} else {
		return NULL;
	}
}

/*
* Get a NLINE when we try and connect 
*/
struct nline *get_nline(char *host)
{
	int i,done=-1;

	if(nlines==NULL) {
		return NULL;
	}
	
	for(i=0;nlines[i]!=NULL && done==-1;i++) {
		if((lc_strncmp(nlines[i]->host,host,-1)==0) ) {
			done=i;
		}
	}

	if(done!=-1) {
		return nlines[done];
	} else {
		return NULL;
	}
}
/*
* Print usafe
*/
static void internal_print_usage()
{
	printf("Chunky Monkey IRCD Version %s\n",VERSION);
	printf("Copyright (c) Chris Plant 2000-2001\n");
	printf("	      David Leadbeater 2001\n");
	printf("	      Mike Saunders 2001\n");
	printf("\n");
	printf("Usage: ircd\n");
#ifdef HAVE_GETOPT_H
	printf(" -p, --port=port	Port to bind to\n");
	printf(" -i, --ip=ip		IP to bind to\n");
	printf(" -c, --config-file=	Config File to use\n");
	printf(" -d, --debug=		Set debug level\n");
	printf(" -f, --foreground	Don't turn into a daemon\n");
	printf(" -h, --help 		Print this message\n");
	printf(" -V, --version		Print version\n");
#else
	printf(" -p <port>		Port to bind to\n");
	printf(" -i <ip>		IP to bind to\n");
	printf(" -c <config_file>	Config File to use\n");
	printf(" -d <debug level>	Set debug level\n");
	printf(" -f			Don't turn into a daemon\n");
	printf(" -h	 		Print this message\n");
	printf(" -V			Print version\n");
#endif
}

/*
 * Print version and GNU license summary
 */
static void internal_print_version()
{
	printf("Chunky Monkey IRCD Version %s\n",VERSION);

}
