/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_kick.c,v 1.9 2002/02/19 23:19:58 lloydy Exp $
 *
 */
#include "chunkyircd.h"
#include "channel.h"



/* Actual routine to kick a user */
static int kick_users(ServerInfo ,ClientInfo ,ChannelInfo ,Cmd );

/* 
* Kick a user 
*/
int cmd_kick(ClientInfo c_ptr,ServerInfo s_ptr,Cmd cmd)
{	
	ChannelInfo chan=NULL;
	ClientInfo client=NULL;
	ChannelMember wrapper=NULL;
	char buffer[BUF_SIZE];

	/* Only Run checks if its a local client */
	if(c_ptr) {

		/* Have we got enough params? */
		if(!check_params(cmd,2)) {
			/* Report not enought params */
			ircd_sprintf(buffer,BUF_SIZE,"%s :Not enough parameters",cmd->command);
			send_numeric(c_ptr,get_my_server(),461,buffer);
			 
			/* Nothing Nasty Happened */
			return 1;
		}

		/* Does the channel exist */
		chan=locate_channel(cmd->params[0]);
		
		if(chan==NULL) {
			/* Invalid channel */
			ircd_sprintf(buffer,BUF_SIZE,"%s :No such channel",cmd->params[0]);
			send_numeric(c_ptr,get_my_server(),403,buffer);
			 
			return 1;
		}

		/* Locate pointer to user wrapper structure */
		wrapper=channel_is_client_on(c_ptr,chan);
	
		if(wrapper==NULL) {
			/* Not on channel */
			ircd_sprintf(buffer,BUF_SIZE,"%s :You're not on that channel",chan->name);
			send_numeric(c_ptr,get_my_server(),442,buffer);
			 
			return 1;
		}

		if( (wrapper->chan_mode & CHANUSR_OP) || 
			(c_ptr->mode & MODE_IRCOP)  ||
			(wrapper->chan_mode & CHANUSR_HALFOP) ) {
			/* They have the correct permissions */
			kick_users(NULL,c_ptr,chan,cmd);
		} else {
			/* They don't have channel ops */
			ircd_sprintf(buffer,BUF_SIZE,"%s :Ha! You're no chan op!",chan->name);
			send_numeric(c_ptr,get_my_server(),482,buffer);
			 
			return 1;
		}
	} else {
		printf("using server nick from %s\n",cmd->origin);
		if(cmd->origin) {
			chan=locate_channel(cmd->params[0]);
			client=locate_client(cmd->origin);
			if(chan && client) {
				kick_users(s_ptr,client,chan,cmd);
			}
		}
	}

	
	/* Return */
	 
	return 1;
}

/* Actually kick users from the channel */
/* All checks been done */
static int kick_users(ServerInfo s_ptr,
		      ClientInfo client,
		      ChannelInfo chan,
	              Cmd cmd)
{
	char buffer[BUF_SIZE];
	ClientInfo kickme=NULL;
	int i;
	ChannelMember wrapper=NULL;
	ChannelMember kicker=NULL;

	if(mass_kicks) {
		for(i=1;cmd->params[i]!=NULL;i++) {
			/* Parse it */
			kickme=locate_client(cmd->params[i]);
			wrapper=channel_is_client_on(kickme,chan);
			kicker=channel_is_client_on(client,chan);
			if(kickme && wrapper && 
			!((wrapper->chan_mode & CHANUSR_OP) && (kicker->chan_mode & CHANUSR_HALFOP) ) ) {
				ircd_sprintf(buffer,BUF_SIZE,"KICK %s %s :%s",chan->name,kickme->nick,cmd->text?cmd->text:"");
				if(client->fd!=-1) 
					send_client_client(client,client,buffer);
				send_to_local_channel(client,chan,buffer);
				servers_relay(s_ptr,client->nick,buffer);

				/* Kick them */
				channel_rem_client(kickme,chan);
				client_rem_chan(kickme,chan);
				/* Close chann if needed */
				channel_needclose(chan);
			}
			wrapper=NULL;
			kickme=NULL;
		}		
			
	} else {
		kickme=locate_client(cmd->params[1]);
		wrapper=channel_is_client_on(kickme,chan);
		if(kickme && wrapper &&
		!((wrapper->chan_mode & CHANUSR_OP) && (kicker->chan_mode & CHANUSR_HALFOP) ) ) {
			
			/* Kick guy in params[1] */
			ircd_sprintf(buffer,BUF_SIZE,"KICK %s %s :%s",chan->name,cmd->params[1],cmd->text?cmd->text:"");
			if(client->fd!=-1) 
				send_client_client(client,client,buffer);
			send_to_local_channel(client,chan,buffer);
			servers_relay(s_ptr,client->nick,buffer);

			/* Remove from lists */
			channel_rem_client(kickme,chan);
			client_rem_chan(kickme,chan);

			/* Close if needed */
			channel_needclose(chan);
		}
	}
	return 1;
}
