/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_notice.c,v 1.12 2002/02/17 23:07:25 lloydy Exp $
 *
 */
#include "chunkyircd.h"
#include "network.h"
#include "channel.h"

/* Utility routine */
static int notice_to_user(ClientInfo,ClientInfo,char *);

/*
* cmd_notice, deals with NOTICE routines
*/
int cmd_notice(ClientInfo c_ptr,ServerInfo s_ptr,Cmd cmd)
{
	ClientInfo recip=NULL;
	ClientInfo sender=NULL;
	ChannelInfo chan=NULL;
	char buffer[BUF_SIZE];
	
	if(!c_ptr) {
		sender=locate_client(cmd->origin);
	} else {
		sender=c_ptr;
	}

	if(!sender) {
		 
		return 0;
	}
	
	/* find out if it's a client or a room */
	if(cmd->params[0]==NULL) {
		ircd_sprintf(buffer,BUF_SIZE,"%s :Not enough parameters\n",cmd->command);
		send_numeric(sender,get_my_server(),461,buffer);
		 
		return 0;
	}
	
	/* Check for masks */
	if(cmd->params[0][0]=='$' && sender->mode & MODE_IRCOP) {
		client_msg_by_server_mask(sender,&(cmd->params[0][1]),"NOTICE",cmd->text,NULL);
		return 1;
	}
	/* if(cmd->params[0][1]=='#' && sender->mode & MODE_IRCOP) {
		client_msg_by_host_mask(sender,&(cmd->params[0][1]),"NOTICE",cmd->text,NULL);
		return 1;
	} */
	
	
	/* Attempt to locate client */
	recip = locate_client(cmd->params[0]);
	if(recip==NULL) {
		if(cmd->params[0][0]=='@') { 
			chan=locate_channel(&(cmd->params[0][1]));
		} else {
			chan=locate_channel(cmd->params[0]);
		}
		if(!chan) {
			ircd_sprintf(buffer,BUF_SIZE,"%s :No such nick/channel",cmd->params[0]);
			send_numeric(sender,get_my_server(),401,buffer);
			 
			return 1;
		} else {
			if(s_ptr || channel_msg_check_perms(chan,sender)) {
				if(ircd_sprintf(buffer,BUF_SIZE,"NOTICE %s%s :%s",cmd->params[0][0]=='@'?"@":"",chan->name,cmd->text)) {
					if(cmd->params[0][0]=='@') {
						send_to_local_channel_ops(sender,chan,buffer);
					} else {
						send_to_local_channel(sender,chan,buffer);
					}
					servers_relay(s_ptr,sender->nick,buffer);
				} 
			} else {
				ircd_sprintf(buffer,BUF_SIZE,"%s :Cannot send to channel",chan->name);
					send_numeric(sender,get_my_server(),404,buffer);
			}
		}
	} else {
		/* Send to a client */
		if(!sender) {
			sender=locate_client(cmd->origin);
		}
		if(sender) {
			if(notice_to_user(sender,recip,cmd->text)==EXIT_SUCCESS) {
				 
				return 1;
			} else {
				 
				return 0;
			}
		}
		
	}
	 
	return 1;
}

static int notice_to_user(ClientInfo sender,
			   ClientInfo recip,
			   char *text)
{
	char cmd_buff[BUF_SIZE];
	
	/* form end of string */
	if(text==NULL) {
		return 0;
	} else {
		ircd_sprintf(cmd_buff,BUF_SIZE,"NOTICE %s :%s",recip->nick,text);
	
		/* call procedure to send to client */
	 	return	send_client_client(recip,sender,cmd_buff);
	}
}

