/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_oper.c,v 1.12 2001/12/26 00:42:44 lloydy Exp $
 *
 */
#include "chunkyircd.h"
#include "network.h"

/* 
 * Issue a user IRC OP status 
 */
int cmd_oper(ClientInfo client,ServerInfo s_ptr,Cmd cmd)
{	
	char buffer[200];
	char host_string[HOST_LENGTH + USER_LENGTH + 5];
	int i;
	int host_match=0,user_match=0,pass_match=0;
	int done=0;

	
	/* Only accept local */
	if(!client || s_ptr)
		return 0;
	
	if(cmd->params[0]==NULL || cmd->params[1]==NULL) {
		/* Return ERR_NEEDMOREPARAMS */
		ircd_sprintf(buffer,200,"%s :Not enough parameters",cmd->command);
		send_numeric(client,get_my_server(),461,buffer);
		 
		return 1;
	}
	
	/* Check irc_ops is defined */
	if(olines==NULL || olines[0]==NULL) {
		send_numeric(client,get_my_server(),
			     464,":Password incorrect");
		 
		return 1;
	}
	
		
	/* extern char *ircops[10,2] contains name/passwd combos */
	/* set in config_file.c on load */
	/* case sensitive */
	for(i=0;olines[i]!=NULL && done==0;i++) {

		if(strcmp(olines[i]->name,cmd->params[0])==0) {
			user_match=1;
			/* Username match */
			
			ircd_sprintf(host_string,HOST_LENGTH+USER_LENGTH+5,"%s@%s",client->user,client->host);
			if(check_string(host_string,olines[i]->hostmask)) {
				host_match=1;
			} else {
				host_match=0;
			}

			/* Password check */
			if(strcmp(olines[i]->pass,cmd->params[1])==0) {
				pass_match=1;
			} else {
				pass_match=0;
			}

			if(pass_match && host_match && user_match) 
				done=1;
		} else {
			user_match=0;
		}
	}


	/* Reply with correct info */
	if( !(user_match) || !(host_match)) {
		/* No o-lines reply */
		log_msg(LOG_BASIC,"Failed OPER attempt (No O-Line) for %s (%s@%s)",client->nick,
										     client->user,
										     client->host);
		send_numeric(client,get_my_server(),491,":No O-lines for your host");
		 
		return 0;
	} 

	if( !(pass_match) ) {
		log_msg(LOG_BASIC,"Failed OPER attempt (Password Incorrect) for %s (%s@%s)",client->nick,
											      client->user,
											      client->host);
		send_numeric(client,get_my_server(),464,":Password incorrect");
		 
		return 0;
	}
	
	/* Give them ops */
	if(!(client->mode & MODE_IRCOP)) {
		/* Give them the Mode, and report the change */
		client->mode |= MODE_IRCOP;
		ircd_sprintf(buffer,200,"MODE %s +o",client->nick);
		send_origin_client(client,client->nick,buffer);
		/* Only update the counter if they're not an OP already */
		ops++;
	}

	/* Tell them they're opped */
	servers_relay(NULL,client->nick,buffer);
	send_numeric(client,get_my_server(),381,":You are now an IRC Operator"); 

	/* Log it */
	log_msg(LOG_BASIC,"Granted Mode +o (IRC Operator) to %s (%s@%s)",client->nick,
									    client->user,
									    client->host);
	
	 
	return 1;
	
}

