/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_ping.c,v 1.7 2002/02/11 20:06:57 lloydy Exp $
 *
 */
#include "chunkyircd.h"
#include "network.h"

CMIRCD_MODULE(0.98,CMD_CLIENT | CMD_SERVER);

/* respond to a ping */
int cmd_ping(ClientInfo client,ServerInfo s_ptr,Cmd cmd)
{
	char buffer[200];
	ServerInfo dest_serv=NULL;
	ClientInfo dest_client=NULL;

	/* if client is non-null its local */
	if(client) {
		/* Check for correct number of paramters */
		if(!check_params(cmd,1)) {
			/* Not enough */
			send_numeric(client,get_my_server(),461,"PING :Not enough parameters");
			 
			return 0;
		} 
		/* Check for target */
		if(!cmd->text || lc_strncmp(cmd->text,server_name,-1)==0) {
			/* No target, or the target is us */
			ircd_sprintf(buffer,200,"PONG %s :%s",server_name,cmd->params[0]);
			send_origin_client(client,server_name,buffer);
		} else {
			/* Target, not for us */
			dest_serv = locate_server(cmd->text);

			if(!dest_serv) {
				/* We don't know that server */
				ircd_sprintf(buffer,200,"%s :No such server",cmd->text);
				send_numeric(client,get_my_server(),402,buffer);
			} else {
				/* We do */
				ircd_sprintf(buffer,200,"PING %s :%s",cmd->params[0],dest_serv->name);
				send_origin_server(dest_serv,client->nick,buffer);
			}
		}

	} else {
		/* From another client via  a server  OR from another server */
		/* Either way, we should have an origin */
		if(!cmd->text || lc_strncmp(cmd->text,server_name,-1)==0) {
			/* Its for us */
			/* try to find a matching server to return to */
			if(!(cmd->origin) || (dest_serv=locate_server(cmd->origin)) ) {
				if(!cmd->origin) {
					ircd_sprintf(buffer,200,"PONG %s :%s",server_name,s_ptr->name);
				} else {
					ircd_sprintf(buffer,200,"PONG %s :%s",server_name,dest_serv->name);
				}
				send_origin_server(cmd->origin?dest_serv:s_ptr,server_name,buffer);
			} else {
				/* Assume its a client */
				ircd_sprintf(buffer,200,"PONG %s :%s",cmd->params[0],cmd->origin);
				if(cmd->origin && (dest_client=locate_client(cmd->origin))) {
					send_origin_client(dest_client,server_name,buffer);
				} else {
					/* Fucked */
				}
			}
		} else {
			/* not for us */
			/* Forward */
			if((dest_serv=locate_server(cmd->text))) {
				ircd_sprintf(buffer,200,"PING %s :%s",s_ptr->name,dest_serv->name);
				send_origin_server(dest_serv,cmd->origin,buffer);
			} else {
				printf("FIXME: Forward a ping to a non-existant server?\n");
			}
		}
	}
	
	 
	return 1;
}


