/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_squit.c,v 1.10 2001/12/31 16:42:13 lloydy Exp $
 *
 */
#include "chunkyircd.h"
#include "server.h"
#include "ircd.h"

/* 
 * cmd_squit, available to opers to force a server connect, or
 * passed by a server to indicate another server has lost the connection
*/ 
int cmd_squit(ClientInfo c_ptr,ServerInfo s_ptr,Cmd cmd)
{
	/* Pointer to server we want to remove */
	ServerInfo d_ptr=NULL;
	int i=0;
	char buffer[BUF_SIZE];
	
	/* If s_ptr is non-null, it relates to a server we don't have a direct
	 * connection to, otherwise, (ie c_ptr!=NULL), we should disconnect the
	 * connection they ask us to */
	

	if(c_ptr) {
		char reason[200];

		/* Check for operators */
		if(!(c_ptr->mode & MODE_IRCOP) ) {
			send_numeric(c_ptr,get_my_server(),481,":Permission Denied  - You're no IRC op");
			 
			return 1;
		}

		/* Check for params */
		if(!(check_params(cmd,1)) || !(cmd->text) ) {
			send_numeric(c_ptr,get_my_server(),461,"SQUIT :Not enough parameters");
			 
			return 1;
		}
		
		/* Check the server exists */
		d_ptr=locate_server(cmd->params[0]);
		if(!(d_ptr) || (d_ptr==get_my_server())) {
			ircd_sprintf(buffer,BUF_SIZE,"%s :No such server",cmd->params[0]);
			send_numeric(c_ptr,get_my_server(),402,buffer);
			 
			return 1;
		}
		
		/* XXX - Only allow local server links to be broken */
		if(d_ptr->fd==-1) {
			send_numeric(c_ptr,get_my_server(),710,":SQUIT Only supported on local links");
			 
			return 1;
		}
		
		/* If its local, sort out the fd */
		if(d_ptr->fd == highest_fd) {
			for(i=d_ptr->fd;i>=0 ;i--) {
				if(fd_list[i].server) {
					highest_fd=i;
					i=-1;
				}
			}
			if(i==0) 
				highest_fd=-1;
		}

		/* Work Out if need to close the link */
		fd_list[d_ptr->fd].server=NULL;
		close(d_ptr->fd);
		
		ircd_sprintf(reason,200,"SQUIT used by %s",c_ptr->nick);
		
		/* Inform other servers */
		server_squit_announce(d_ptr,reason);
		
		/* Work out clients on that server */
		clients_netsplit(d_ptr,d_ptr->name);
		
		/* Generate SQUIT's for servers behind that server */
		/* it works out clients and announces */
		server_squit_generate(d_ptr,reason,d_ptr->name);
		
		/* Cleanup the hash and memory */
		rem_server_hash(d_ptr->name);
		get_my_server()->server_count--;
		free(d_ptr);

	} else {
		d_ptr=locate_server(cmd->params[0]);
		if(d_ptr) {
			
			d_ptr->via_ptr->server_count--;

			if(d_ptr->fd!=-1) {
				fd_list[d_ptr->fd].server=NULL;
				close(d_ptr->fd);
			}

			if(d_ptr->fd == highest_fd) {
				for(i=d_ptr->fd;i>=0 ;i--) {
					if(fd_list[i].server || fd_list[i].client || fd_list[i].temp) {
						highest_fd=i;
						i=-1;
					}
				}
				if(i==0) 
					highest_fd=-1;
			}
			

			/* Inform other servers */
			server_squit_announce(d_ptr,cmd->text);
		
			/* Work out clients on that server */
			clients_netsplit(d_ptr,d_ptr->name);
		
			/* Work out which clients and servers behind this one where disconnected */
			server_squit_generate(d_ptr,cmd->text,d_ptr->name);

			/* Tell other servers we netsplitted from this server, and they work out the rest */
			ircd_sprintf(buffer,BUF_SIZE,"SQUIT %s :%s",d_ptr->name,cmd->text);
			servers_relay(d_ptr,cmd->origin?cmd->origin:get_my_server()->name,buffer);
			
			rem_server_hash(d_ptr->name);

			free(d_ptr);
		
			return 1;
		} else {
			printf("Recived a non-existant server QUIT\n");
			return 0;

		}

	}

	return 0;
}

		

