/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_whois.c,v 1.13 2002/02/19 23:19:58 lloydy Exp $
 *
 */
#include "chunkyircd.h"
#include "network.h"
#include "client.h"
#include "channel.h"

int cmd_whois(ClientInfo sender,ServerInfo s_ptr,Cmd cmd)
{
	ClientInfo user_about=NULL;
	char buffer[BUF_SIZE];
	char frm_nick[CHANNEL_MAX_LENGTH+3];
	int i;
	ChannelMember user_chan=NULL;
	ChannelMember sender_chan=NULL;
	
	if(!sender || s_ptr)
		return 0;
	
	if(cmd->params[0]==NULL) {
		/* Send error reply */
		send_numeric(sender,get_my_server(),431,":No nickname given");
		 
		return 1;
	}
	
	user_about=locate_client(cmd->params[0]);
		
	/* Return to sender, with correct reply */
	if(user_about!=NULL) {
		/* General Info */
		ircd_sprintf(buffer,BUF_SIZE,"%s %s %s * :%s",user_about->nick,
					      user_about->user,
					      user_about->host,
					      user_about->real_name);
		send_numeric(sender,get_my_server(),311,buffer);
		
		/* Send Channel list */
		if(user_about->chan_list && user_about->chan_list[0]) {
			ircd_sprintf(buffer,BUF_SIZE,"%s :",user_about->nick);
			for(i=0;i<user_about->chanlist_no && user_about->chan_list[i]!=NULL;i++) {
				/* See if we have chanops */
				user_chan=channel_is_client_on(user_about,user_about->chan_list[i]);
				sender_chan=channel_is_client_on(sender,user_about->chan_list[i]);
				/* Don't list channels with mode +s */
				if(!(user_about->chan_list[i]->mode & CMODE_SECRET) || (sender_chan) ) {
					ircd_sprintf(frm_nick,CHANNEL_MAX_LENGTH+2,"%s%s ",
						(user_chan->chan_mode & CHANUSR_OP)?"@":
						(user_chan->chan_mode & CHANUSR_HALFOP)?"%":
						(user_chan->chan_mode & CHANUSR_VOICE)?"+":"",
						user_about->chan_list[i]->name);
					strcat(buffer,frm_nick);
				}
			}
			send_numeric(sender,get_my_server(),319,buffer);
		}
			
		/* Server info */
		ircd_sprintf(buffer,BUF_SIZE,"%s %s :%s",user_about->nick,user_about->server->name,
							 user_about->server->info);
		send_numeric(sender,get_my_server(),312,buffer);
		
		/* Inform if irc op */
		if( (user_about->mode & MODE_IRCOP) ) {
			ircd_sprintf(buffer,BUF_SIZE,"%s :Is an IRC Operator",user_about->nick);
			send_numeric(sender,get_my_server(),313,buffer);
		}
		
		/* If they are away */
		if( (user_about->mode & MODE_AWAY) ) {
			ircd_sprintf(buffer,BUF_SIZE,"%s :%s",user_about->nick,user_about->away_msg);
			send_numeric(sender,get_my_server(),301,buffer);
		}
		
		/* Idle time */
		ircd_sprintf(buffer,BUF_SIZE,"%s %ld :seconds idle",user_about->nick,
					 	     time(NULL)-(user_about->idle_timer));
		send_numeric(sender,get_my_server(),317,buffer);
		
		/* Inform end of whois */
		ircd_sprintf(buffer,BUF_SIZE,"%s :End of WHOIS List",user_about->nick);
		send_numeric(sender,get_my_server(),318,buffer);
		

	} else {
		/* Send error reply */
		sprintf(buffer,"%s :No such nick/channel",cmd->params[0]);
		send_numeric(sender,get_my_server(),401,buffer);	
	}
	 
	return 1;
}

