#!/usr/bin/perl -w
# For testing cmircd - run it like perl cmtest.pl 10 > /tmp/log
# it puts all input/output in /tmp/log when run like that

# $Id: cmtest.pl,v 1.1 2001/12/24 12:17:15 lloydy Exp $

# $Log: cmtest.pl,v $
# Revision 1.1  2001/12/24 12:17:15  lloydy
#
# Updates to debugging code
# Added Test Script
#
# Revision 1.13  2001/12/22 14:43:20  dgl
# added kick, pong ping fixed a few things
#
# Revision 1.12  2001/10/26 18:15:40  dgl
# added random
#
# Revision 1.11  2001/10/26 17:16:48  dgl
# added more stuff, nul, colons, comma
#
# Revision 1.10  2001/10/26 16:52:54  dgl
# quit -> last
#
# Revision 1.9  2001/10/26 10:16:24  dgl
# Added option checking and use different ports
#
# Revision 1.8  2001/10/24 15:51:09  dgl
# Fixed to respond to pings, better logging and documentation (two comments :)
#
# Revision 1.7  2001/10/24 15:04:01  dgl
# rcs..
#
# Revision 1.6  2001/10/24 15:03:26  dgl
# playing with rcs :)
#
use strict;
use IO::Select;
use IO::Socket;
$|++;

my @queue;
my @clients;
my $sel = new IO::Select;
my @cmds = qw/NICK USER KILL OPER JOIN KICK PING PONG PART PRIVMSG NOTICE MODE LIST NAMES TOPIC LUSERS WHO MOTD CONNECT SQUIT WHOIS QUIT/;

my %stuff = (
    vaild_channel => sub { return '#monkeyircd' },
	vaild_user => sub { return 'dg' },
	invaild_channel => sub { return '#fdjjjdjdjdj'},
	invaild_user => sub { return 'fdkslfkdslfksdlsdk' },
	garbage => sub{ 
	   open(RAND, "/dev/urandom");my $tmp;read(RAND,$tmp,100);close(RAND);
	   $tmp =~ s/\n//gs;
	   return $tmp;
	},
	random => sub { return makerand() },
	len16 => sub { return 'a' x 16 },
	len31 => sub { return 'b' x 31 },
	len63 => sub { return 'c' x 63 },
	len99 => sub { return 'd' x 99 },
	nul => sub { return "\0"},
	spaces => sub { return ' ' x 20 },
	colon => sub {return ' :'},
	colons => sub { return ':::::::::::::'},
	comma => sub { return ','},
);

if(!$ARGV[0] or $ARGV[0] =~ /\D/) {
   note("You must provide the number of clients to load on the command line");
   exit;
}

note("Making outputs..");

for my $cmd(@cmds) {
   for(qw/len16 len31 len63 len99 vaild_channel vaild_user invaild_user invaild_channel garbage nul spaces colon colons comma random/){
      addqueue("$cmd ".&{$stuff{$_}} . "\r\n");
      addqueue("$cmd :".&{$stuff{$_}} . "\r\n");
   }

   for my $first(qw/len16 len31 len63 len99 vaild_channel vaild_user invaild_user invaild_channel garbage nul spaces colon colons comma random/) {
	  for(qw/len16 len31 len63 len99 vaild_channel vaild_user invaild_user invaild_channel garbage nul spaces colon colons comma random/) {
		 addqueue("$cmd ".&{$stuff{$first}} . ' ' . &{$stuff{$_}} . "\r\n");
		 addqueue("$cmd ".&{$stuff{$first}} . ' :' . &{$stuff{$_}} . "\r\n");
		 addqueue("$cmd :".&{$stuff{$first}} . ' ' . &{$stuff{$_}} . "\r\n");
	  }
   }
}

note("Outputs queued, loading clients...");
for(0..$ARGV[0]) {
   newclient();
}
note("Clients loaded, out of " . ($ARGV[0] + 1) ." there were " . scalar @clients . " successful connections");
die("No connections") if scalar @clients < 1;

note("Begining run, brace yourself");
my $start = time;
my $i=0;
my $c=scalar @clients;

while(1) {
   my @ready = $sel->can_read(2.2);
   foreach my $fh (@ready){
		 my($tmp,$char);
		 while (sysread($fh, $char, 1)) { $tmp .= $char; last if $char eq "\n"}
		 closefh($fh) if !defined $tmp || !length $tmp;
		 print "<< $tmp" if defined $tmp;
		 if(defined $tmp && $tmp =~ s/^PING/PONG/){
			syswrite($fh, $tmp."\r\n");
		 }
   }
   if(!@ready) {
      for(@clients) {
         sendfh($_, getqueue());
      }
      $i++;
      note("Pass $i done, still " . scalar @queue . " items in the queue");
   }
   last if $c < 1;
   last if !@queue;
}

if($c < 1) {
   note("No clients left, either the server died or we flooded ourselves off, runtime was: ".(time - $start) . " seconds");
}else{
   note("Finished, this run took " . (time - $start) . " seconds");
}

sub note {
   print STDERR '[' . (scalar localtime) . '] ' . $_[0] . "\n";
}

sub addqueue {
   push(@queue,$_[0]);
}

sub getqueue {
   return shift @queue;
}

sub sendfh {
   my($fh,$text) = @_;
   print ">> $text";
   syswrite($fh,$text,length $text) or closefh($fh);
}

sub closefh {
   my $fh = shift;
   close($fh);
   $sel->remove($fh);
   note("Lost connection to $fh");
   @clients = map(!$fh, @clients);
   $c--;
}

sub newclient {
   my $irc = new IO::Socket::INET(
		 PeerAddr => $ARGV[2] || 'localhost',
		 PeerPort => $ARGV[1] || 6667)
     or print "Unable to connect to server: $!\n";
   return if not defined $irc;
   $sel->add($irc);
   push(@clients, $irc);
   syswrite($irc,"NICK ".makerand()."\r\nUSER ".makerand()." ".makerand()." ".makerand()." :".makerand()."\r\n");
}

sub makerand {
   my @stuff = 'a' .. 'z';
   my $tmp = $stuff[rand $#stuff];
   push(@stuff, '0' .. '9' , '`' , '_' ,'-','^','|','{','}','[',']','\\');
   for(0..(int(rand(15)))) {
	  $tmp .= $stuff[rand $#stuff];
   }
   return $tmp;
}

